#!/usr/bin/env python
# coding=utf-8
# Created by: Li Yao (ly349@cornell.edu)
# Created on: 21/19/20
import argparse
import sys
import os
try:
    import pyBigWig
    import pandas as pd
    import numpy as np
except ImportError:
    sys.exit("Please get both pyBigWig installed first!")


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("-i", "--input", help="Input bed file", required=True)
    parser.add_argument("-p", "--pl-bw", help="Bigwig track file for signals mapped to the forward strand",
                        required=True)
    parser.add_argument("-m", "--mn-bw", help="Bigwig track file for signals mapped to the reverse strand",
                        required=True)
    parser.add_argument("--single", action="store_true", default=False, required=False,
                        help="Set this switch to allow unidirectional TREs")
    parser.add_argument("-s", "--save-to", help="Save TSSs bed to ...", required=True)

    args = parser.parse_args()

    if os.path.exists(args.pl_bw) and os.path.exists(args.mn_bw) and os.path.exists(args.input):
        pl_track = pyBigWig.open(args.pl_bw)
        mn_track = pyBigWig.open(args.mn_bw)

        peak_df = pd.read_csv(args.input, sep="\t", header=None)
        if peak_df.shape[1] >= 3:
            peak_df["pl_TSS"] = -1
            peak_df["mn_TSS"] = -1
            for nr, row in peak_df.iterrows():
                try:
                    pl_tss = np.nanargmax(pl_track.values(row[0], row[1], row[2])) + row[1]
                except:
                    pl_tss = -1
                if pl_tss == -1 and not args.single:
                    continue
                peak_df.loc[nr, "pl_TSS"] = pl_tss

                try:
                    mn_tss = np.nanargmin(mn_track.values(row[0], row[1], row[2])) + row[1]
                except:
                    mn_tss = -1
                if mn_tss == -1 and not args.single:
                    continue
                peak_df.loc[nr, "mn_TSS"] = mn_tss

            peak_df["name"] = "."
            peak_df = peak_df.loc[np.logical_and(peak_df.pl_TSS >= 0,
                                                 peak_df.mn_TSS >= 0),
                                  (0, 1, 2, "name", "pl_TSS", "mn_TSS")]
            peak_df.to_csv(args.save_to, sep="\t", index=False, header=False)
        else:
            print("There must be at least 3 columns in the bed file!")
